#include "SpriteSystem.h"

void InitSprite(Sprite* sprite,
UINT8 id, UINT8 mem, UINT8 x, UINT8 y)
{
    sprite->id = mem;
    set_sprite_tile(mem, id);
    sprite->rect.width = SPRITE_SIZE;
    sprite->rect.height = SPRITE_SIZE;
    set_sprite_prop(mem, get_sprite_prop(mem) & ~S_FLIPX & ~S_FLIPY);
    MoveSprite(sprite, x, y);
}

void InitMetaSprite1x2(MetaSprite1x2* metaSprite1x2,
UINT8 id1, UINT8 mem, UINT8 x, UINT8 y)
{
    metaSprite1x2->ids[0] = mem;
    metaSprite1x2->ids[1] = mem+1;
    set_sprite_tile(mem, id1);
    set_sprite_tile(mem+1, id1+1);
    metaSprite1x2->rect.width = SPRITE_SIZE;
    metaSprite1x2->rect.height = SPRITE_SIZE2;
    set_sprite_prop(mem, get_sprite_prop(mem) & ~S_FLIPX & ~S_FLIPY);
    set_sprite_prop(mem+1, get_sprite_prop(mem+1) & ~S_FLIPX & ~S_FLIPY);
    MoveMetaSprite1x2(metaSprite1x2, x, y);
}

void InitMetaSprite2x1(MetaSprite2x1* metaSprite2x1,
UINT8 id1, UINT8 mem, UINT8 x, UINT8 y)
{
    metaSprite2x1->ids[0] = mem;
    metaSprite2x1->ids[1] = mem+1;
    set_sprite_tile(mem, id1);
    set_sprite_tile(mem+1, id1+1);
    metaSprite2x1->rect.width = SPRITE_SIZE2;
    metaSprite2x1->rect.height = SPRITE_SIZE;
    set_sprite_prop(mem, get_sprite_prop(mem) & ~S_FLIPX & ~S_FLIPY);
    set_sprite_prop(mem+1, get_sprite_prop(mem+1) & ~S_FLIPX & ~S_FLIPY);
    MoveMetaSprite2x1(metaSprite2x1, x, y);
}

void InitMetaSprite2x2(MetaSprite2x2* metaSprite2x2,
UINT8 id1, UINT8 mem, UINT8 x, UINT8 y)
{
    for(i = 0; i < 4; ++i)
    {
        metaSprite2x2->ids[i] = mem + i;
        set_sprite_tile(mem + i, id1 + i);
        set_sprite_prop(mem + i, get_sprite_prop(mem + i) & ~S_FLIPX & ~S_FLIPY);
    }
    metaSprite2x2->rect.width = SPRITE_SIZE2;
    metaSprite2x2->rect.height = SPRITE_SIZE2;
    MoveMetaSprite2x2(metaSprite2x2, x, y);
}

void InitMetaSprite4x4(MetaSprite4x4* metaSprite4x4,
UINT8 id1, UINT8 mem, UINT8 x, UINT8 y)
{
    for(i = 0; i < 16; ++i)
    {
        metaSprite4x4->ids[i] = mem + i;
        set_sprite_tile(mem + i, id1 + i);
        set_sprite_prop(mem + i, get_sprite_prop(mem + i) & ~S_FLIPX & ~S_FLIPY);
    }
    metaSprite4x4->rect.width = SPRITE_SIZE4;
    metaSprite4x4->rect.height = SPRITE_SIZE4;
    MoveMetaSprite4x4(metaSprite4x4, x, y);
}

void MoveSprite(Sprite* sprite, UINT8 x, UINT8 y)
{
    sprite->rect.x = x;
    sprite->rect.y = y;
    move_sprite(sprite->id, x, y);
}

void MoveMetaSprite1x2(MetaSprite1x2* metaSprite1x2, UINT8 x, UINT8 y)
{
    metaSprite1x2->rect.x = x;
    metaSprite1x2->rect.y = y;
    move_sprite(metaSprite1x2->ids[0], x, y);
    move_sprite(metaSprite1x2->ids[1], x, y + SPRITE_SIZE);
}

void MoveMetaSprite1x2FlipY(MetaSprite1x2* metaSprite1x2, UINT8 x, UINT8 y)
{
    metaSprite1x2->rect.x = x;
    metaSprite1x2->rect.y = y;
    move_sprite(metaSprite1x2->ids[1], x, y);
    move_sprite(metaSprite1x2->ids[0], x, y + SPRITE_SIZE);
}

void MoveMetaSprite2x1(MetaSprite2x1* metaSprite2x1, UINT8 x, UINT8 y)
{
    metaSprite2x1->rect.x = x;
    metaSprite2x1->rect.y = y;
    move_sprite(metaSprite2x1->ids[0], x, y);
    move_sprite(metaSprite2x1->ids[1], x + SPRITE_SIZE, y);
}

void MoveMetaSprite2x1FlipX(MetaSprite2x1* metaSprite2x1, UINT8 x, UINT8 y)
{
    metaSprite2x1->rect.x = x;
    metaSprite2x1->rect.y = y;
    move_sprite(metaSprite2x1->ids[1], x, y);
    move_sprite(metaSprite2x1->ids[0], x + SPRITE_SIZE, y);
}

void MoveMetaSprite2x2(MetaSprite2x2* metaSprite2x2, UINT8 x, UINT8 y)
{
    metaSprite2x2->rect.x = x;
    metaSprite2x2->rect.y = y;
    move_sprite(metaSprite2x2->ids[0], x, y);
    move_sprite(metaSprite2x2->ids[1], x + SPRITE_SIZE, y);
    move_sprite(metaSprite2x2->ids[2], x, y + SPRITE_SIZE);
    move_sprite(metaSprite2x2->ids[3], x + SPRITE_SIZE, y + SPRITE_SIZE);
}

void MoveMetaSprite2x2FlipY(MetaSprite2x2* metaSprite2x2, UINT8 x, UINT8 y)
{
    metaSprite2x2->rect.x = x;
    metaSprite2x2->rect.y = y;
    move_sprite(metaSprite2x2->ids[2], x, y);
    move_sprite(metaSprite2x2->ids[3], x + SPRITE_SIZE, y);
    move_sprite(metaSprite2x2->ids[0], x, y + SPRITE_SIZE);
    move_sprite(metaSprite2x2->ids[1], x + SPRITE_SIZE, y + SPRITE_SIZE);
}

void MoveMetaSprite2x2FlipX(MetaSprite2x2* metaSprite2x2, UINT8 x, UINT8 y)
{
    metaSprite2x2->rect.x = x;
    metaSprite2x2->rect.y = y;
    move_sprite(metaSprite2x2->ids[1], x, y);
    move_sprite(metaSprite2x2->ids[0], x + SPRITE_SIZE, y);
    move_sprite(metaSprite2x2->ids[3], x, y + SPRITE_SIZE);
    move_sprite(metaSprite2x2->ids[2], x + SPRITE_SIZE, y + SPRITE_SIZE);
}

void MoveMetaSprite4x4(MetaSprite4x4* metaSprite4x4, UINT8 x, UINT8 y)
{
    metaSprite4x4->rect.x = x;
    metaSprite4x4->rect.y = y;
    move_sprite(metaSprite4x4->ids[0], x, y);
    move_sprite(metaSprite4x4->ids[1], x + SPRITE_SIZE, y);
    move_sprite(metaSprite4x4->ids[2], x + SPRITE_SIZE2, y);
    move_sprite(metaSprite4x4->ids[3], x + SPRITE_SIZE3, y);
    move_sprite(metaSprite4x4->ids[4], x, y + SPRITE_SIZE);
    move_sprite(metaSprite4x4->ids[5], x + SPRITE_SIZE, y + SPRITE_SIZE);
    move_sprite(metaSprite4x4->ids[6], x + SPRITE_SIZE2, y + SPRITE_SIZE);
    move_sprite(metaSprite4x4->ids[7], x + SPRITE_SIZE3, y + SPRITE_SIZE);
    move_sprite(metaSprite4x4->ids[8], x , y + SPRITE_SIZE2);
    move_sprite(metaSprite4x4->ids[9], x  + SPRITE_SIZE, y + SPRITE_SIZE2);
    move_sprite(metaSprite4x4->ids[10], x + SPRITE_SIZE2, y + SPRITE_SIZE2);
    move_sprite(metaSprite4x4->ids[11], x + SPRITE_SIZE3, y + SPRITE_SIZE2);
    move_sprite(metaSprite4x4->ids[12], x, y + SPRITE_SIZE3);
    move_sprite(metaSprite4x4->ids[13], x + SPRITE_SIZE, y + SPRITE_SIZE3);
    move_sprite(metaSprite4x4->ids[14], x + SPRITE_SIZE2, y + SPRITE_SIZE3);
    move_sprite(metaSprite4x4->ids[15], x + SPRITE_SIZE3, y + SPRITE_SIZE3);
}

void MoveMetaSprite4x4FlipX(MetaSprite4x4* metaSprite4x4, UINT8 x, UINT8 y)
{
    metaSprite4x4->rect.x = x;
    metaSprite4x4->rect.y = y;
    move_sprite(metaSprite4x4->ids[3], x, y);
    move_sprite(metaSprite4x4->ids[2], x + SPRITE_SIZE, y);
    move_sprite(metaSprite4x4->ids[1], x + SPRITE_SIZE2, y);
    move_sprite(metaSprite4x4->ids[0], x + SPRITE_SIZE3, y);
    move_sprite(metaSprite4x4->ids[7], x, y + SPRITE_SIZE);
    move_sprite(metaSprite4x4->ids[6], x + SPRITE_SIZE, y + SPRITE_SIZE);
    move_sprite(metaSprite4x4->ids[5], x + SPRITE_SIZE2, y + SPRITE_SIZE);
    move_sprite(metaSprite4x4->ids[4], x + SPRITE_SIZE3, y + SPRITE_SIZE);
    move_sprite(metaSprite4x4->ids[11], x , y + SPRITE_SIZE2);
    move_sprite(metaSprite4x4->ids[10], x  + SPRITE_SIZE, y + SPRITE_SIZE2);
    move_sprite(metaSprite4x4->ids[9], x + SPRITE_SIZE2, y + SPRITE_SIZE2);
    move_sprite(metaSprite4x4->ids[8], x + SPRITE_SIZE3, y + SPRITE_SIZE2);
    move_sprite(metaSprite4x4->ids[15], x, y + SPRITE_SIZE3);
    move_sprite(metaSprite4x4->ids[14], x + SPRITE_SIZE, y + SPRITE_SIZE3);
    move_sprite(metaSprite4x4->ids[13], x + SPRITE_SIZE2, y + SPRITE_SIZE3);
    move_sprite(metaSprite4x4->ids[12], x + SPRITE_SIZE3, y + SPRITE_SIZE3);
}
